package se.liu.ida.tdde46;

import static io.restassured.RestAssured.*;
import static org.hamcrest.Matchers.*;
import static org.testng.Assert.*;

import java.util.ArrayList;
import java.util.Map;

import org.apache.commons.lang3.RandomStringUtils;
import org.testng.annotations.Test;

import org.joda.time.DateTime;

import io.restassured.http.ContentType;
import io.restassured.response.Response;

public class PermissionsTest {

	final static String HOST_NAME = "http://localhost:8080";

	@Test
	public void testGetPermissions() {
		Response response = get(HOST_NAME + "/permissions");
		response.then().body("", isA(ArrayList.class));
	}

	@Test
	public void testAddedPermissionExistsInPermissionsList() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		String activityName = RandomStringUtils.randomAlphanumeric(20);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			put(HOST_NAME + "/principals/" + principalName);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"" + principalName + "\", \"activityName\": \"" + activityName + "\"}").
			post(HOST_NAME + "/permissions/create");

		Map<?,?> permissionObject = getPermissionObject(get(HOST_NAME + "/permissions"), activityName, principalName); 
		assertNotNull(permissionObject);
		assertNull(permissionObject.get("expirationDate"));
	}
	
	@Test
	public void testAddedPermissionReturns200Code() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		String activityName = RandomStringUtils.randomAlphanumeric(20);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			put(HOST_NAME + "/principals/" + principalName);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"" + principalName + "\", \"activityName\": \"" + activityName + "\"}").
			post(HOST_NAME + "/permissions/create").
		then().
			statusCode(200);
	}
	
	@Test
	public void testTimeConstrainedPermissionHasExpirationDateLaterThanNow() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		String activityName = RandomStringUtils.randomAlphanumeric(20);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			put(HOST_NAME + "/principals/" + principalName);

		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"" + principalName + "\", \"activityName\": \"" + activityName + "\", \"durationInSeconds\": 50}").
			post(HOST_NAME + "/permissions/create");

		Map<?, ?> permissionObject = getPermissionObject(get(HOST_NAME + "/permissions"), activityName, principalName); 
		assertNotNull(permissionObject);
		assertNotNull(permissionObject.get("expirationDate"));
		DateTime expirationDate = new DateTime((Long)(permissionObject.get("expirationDate")));
		assertTrue(expirationDate.isAfterNow());
	}

	@Test
	public void testMissingActivityNameCausesPermissionCreationToFail() {
		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"IDontMatter\"}").
			post(HOST_NAME + "/permissions/create").
		then().
			statusCode(400);
	}
	
	@Test
	public void testMissingPrincipalNameCausesPermissionCreationToFail() {
		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"activityName\": \"IDontMatter\"}").
			post(HOST_NAME + "/permissions/create").
		then().
			statusCode(400);
	}
	
	@Test
	public void testUnrecognizedPrincipalNameCausesPermissionCreationToFail() {
		String principalName = RandomStringUtils.randomAlphanumeric(20);
		
		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"" + principalName + "\", \"activityName\": \"IDontMatter\"}").
			post(HOST_NAME + "/permissions/create").
		then().
			statusCode(400);
	}
	
	@Test
	public void testMalformedJsonObjectCausesPermissionCreationToFail() {
		given().
			contentType(ContentType.JSON).
			accept(ContentType.JSON).
			body("{\"principalName\": \"IDontMatter\", \"activityName\": \"IDontMatter\"").
			post(HOST_NAME + "/permissions/create").
		then().
			statusCode(400);
	}
	
	private Map<?, ?> getPermissionObject(Response response, String activityName, String principalName) {
		for(Object o : response.jsonPath().getList("")) {
			if(o instanceof Map<?, ?>) {
				Map<?, ?> map = (Map<?, ?>)o;
				if(map.get("activityName").equals(activityName) && map.get("principalName").equals(principalName))
					return map;
			}
		}
		return null;
	}
}
